--[[----------------------------------------------------------------------------

ViewbookAPI.lua
Common code to initiate Viewbook API requests

--------------------------------------------------------------------------------

VIEWBOOK B.V.
 Copyright 2012-2008 Viewbook B.V.
 All Rights Reserved.



------------------------------------------------------------------------------]]

	-- Lightroom SDK
local LrBinding = import 'LrBinding'
local LrDialogs = import 'LrDialogs'
local LrErrors = import 'LrErrors'
local LrFunctionContext = import 'LrFunctionContext'
local LrHttp = import 'LrHttp'
local LrMD5 = import 'LrMD5'
local LrPathUtils = import 'LrPathUtils'
local LrTasks = import 'LrTasks'
local LrView = import 'LrView'
local LrXml = import 'LrXml'
local prefs = import 'LrPrefs'.prefsForPlugin()
local bind = LrView.bind
local share = LrView.share

--[[
local logger = import 'LrLogger'( 'ViewbookAPILog' )
local debug, info, warn, err = logger:quick( 'debug', 'info', 'warn', 'err' )
--Viewbook
logger:enable( "logfile" ) 
--]]

--============================================================================--
--VIEWBOOK API KEY / ENVIRONMENT
prefs.api_key = "7e7d659894e99d9f5e1d830a266a97d7"
prefs.secret = "7ef15a4239529709"
prefs.toplevel = "com"

--============================================================================--

ViewbookAPI = {}

--------------------------------------------------------------------------------

local simpleXmlMetatable = {
	__tostring =	-- This isn't supposed to be picked up by LuaDocs
		function( self ) return self._value end
}

--------------------------------------------------------------------------------

local traverse

traverse = function( node )

	local type = string.lower( node:type() )

	if type == "element" then
		local element = setmetatable( {}, simpleXmlMetatable )
		element._name = node:name()
		element._value = node.text and node:text() or nil
		
		local count = node:childCount()
		for i = 1,count do
			local name, value = traverse( node:childAtIndex( i ) )
			if name and value then
				element[ name ] = value
			end
		end

		if type == 'element' then
			for k,v in pairs( node:attributes() ) do
				element[ k ] = v.value
			end
		end
		
		return element._name, element
	end

end

--------------------------------------------------------------------------------

local function xmlElementToSimpleTable( xmlString )
	local name, value = traverse( LrXml.parseXml( xmlString ) )
	return value
end

--------------------------------------------------------------------------------

--Callback on login
function ViewbookAPI.getApiKeyAndSecret()
	local apiKey = prefs.api_key
	local sharedSecret = prefs.secret
	local token = prefs.token
	return apiKey, sharedSecret, token
end

--------------------------------------------------------------------------------

function ViewbookAPI.makeApiSignature( params )

	-- If no API key, add it in now.
	
	local apiKey, sharedSecret = ViewbookAPI.getApiKeyAndSecret()
	
	if not params.api_key then
		params.api_key = apiKey
	end

	-- Get list of arguments in sorted order.

	local argNames = {}
	for name in pairs( params ) do
		table.insert( argNames, name )
	end
	
	table.sort( argNames )

	-- Build the secret string to be MD5 hashed.
	
	local allArgs = sharedSecret
	for _, name in ipairs( argNames ) do
		if params[ name ] then  -- might be false
			allArgs = string.format( '%s%s%s', allArgs, name, params[ name ] )
		end
	end
	
	-- MD5 hash this string.
	return LrMD5.digest( allArgs )

end


--------------------------------------------------------------------------------

function ViewbookAPI.uploadPhoto( params )
	--info('Upload LOG:', params.photoset_id)
	-- Prepare to upload.
	assert( type( params ) == 'table', 'ViewbookAPI.uploadPhoto: params must be a table' )
	local postUrl = params.photo_id and 'http://api.viewbook.'..prefs.toplevel..'/services/replace/' or 'http://api.viewbook.'..prefs.toplevel..'/services/upload/'

	-- info( 'uploading photo', params.filePath )
	local filePath = assert( params.filePath )
	params.filePath = nil
	local fileName = LrPathUtils.leafName( filePath )

	params.auth_token = params.auth_token or prefs.token
	params.api_sig = ViewbookAPI.makeApiSignature( params )

	local mimeChunks = {}
	for argName, argValue in pairs( params ) do
		if argName ~= 'api_sig' then
			mimeChunks[ #mimeChunks + 1 ] = { name = argName, value = argValue }
		end
	end

	mimeChunks[ #mimeChunks + 1 ] = { name = 'api_sig', value = params.api_sig }
	mimeChunks[ #mimeChunks + 1 ] = { name = 'photo', fileName = fileName, filePath = filePath, contentType = 'application/octet-stream' }
	
	-- Post it and wait for confirmation.
	local result = LrHttp.postMultipart( postUrl, mimeChunks )

	--info( 'uploading photo DONE', result)

	-- Parse Viewbook response for photo ID.
	local simpleXml = xmlElementToSimpleTable( result )
	if simpleXml.stat == 'ok' then
		return simpleXml.photoid._value
	else
		LrErrors.throwUserError( LOC( "$$$/Viewbook/Error/API/Upload=Viewbook API returned an error message (function upload, message ^1)",
							tostring( simpleXml.err and simpleXml.err.msg ) ) )
	end

end


--------------------------------------------------------------------------------
-- Request the photoset list
function ViewbookAPI.getPhotoSets(propertyTable)

	propertyTable.loading = true;
	params = {}
	params.method = 'flickr.photosets.getList' 

	-- Automatically add API key.
	local apiKey, sharedSecret = ViewbookAPI.getApiKeyAndSecret()
	
	if not params.api_key then
		params.api_key = apiKey
	end
	if prefs.token then
		params.auth_token = prefs.token
	end
	
	-- Build up the URL for this function.
	params.api_sig = ViewbookAPI.makeApiSignature( params )
	local url = string.format( 'http://api.viewbook.'..prefs.toplevel..'/services/rest/?method=%s', assert( params.method ) )
	
	
	for name, value in pairs( params ) do
		if name ~= 'method' and value then  -- the 'and value' clause allows us to ignore false
			-- URL encode each of the params.
			value = string.gsub( value, '([^ !#-$&-*,%-%.0-;@-~])', function( c ) return string.format( '%%%02X', string.byte( c ) ) end )
			value = string.gsub( value, ' ', '+' )
			params[ name ] = value
			url = string.format( '%s&%s=%s', url, name, value )
		end
	end
	--info('URL: ',url)
	-- Call the URL and wait for response.

	LrFunctionContext.postAsyncTaskWithContext( 'get task ',
		function( context )
			local response = LrHttp.get(url)

			--info ('API response XX', response )

			-- All responses are XML. Parse it now.
			local simpleXml = xmlElementToSimpleTable( response )
			if simpleXml.stat == 'ok' then
				
				--info( 'Viewbook API returned status OK' )
				-- Parse Viewbook response for photoSets
			    local photsetXML = LrXml.parseXml(response)
			  	local photoSetsNodes = photsetXML:childAtIndex(1)
				local NumberOfPhotoSets = photoSetsNodes:childCount()

				--  Create items table style
				--	local items = {
				--		{ title = "Willie Nelson Picnic '07", value = 'willie' },
				--		{ title = "Family Pictures", value = 'family' },
				--		{ title = "Iceland '03", value = 'iceland' },
				--	}
				local items = {}
				-- loop through
				for count = 1,NumberOfPhotoSets do 
					local ele = photoSetsNodes:childAtIndex(count)
					-- info('DEBUG XML',ele)
					local attr = ele:attributes()
					local photoSetId = tostring(attr["id"]["value"])
					-- info('DEBUG ID',photoSetId)
					local photoSetTitle = tostring(ele:childAtIndex(1):childAtIndex(1):serialize())
					-- info('DEBUG XML TITLE',photoSetTitle)

					-- build items table
					items[count] = {title = photoSetTitle, value = photoSetId}
				end  
			    -- info('DEBUG ITEMS',items[1]["title"])
				--return items
				propertyTable.photosets = items
				
				propertyTable.loading = false;				
			else
				warn( 'Viewbook API returned error', simpleXml.err and simpleXml.err.msg )
				LrErrors.throwUserError( LOC( "$$$/Viewbook/Error/API=Viewbook API returned an error message (function ^1, message ^2)",
									tostring( params.method ),
									tostring( simpleXml.err and simpleXml.err.msg ) ) )
			end
		end)
end


--------------------------------------------------------------------------------
--
-- Given a string, return the urlencoded version of it
--
function urlencode(text)
   return text:gsub('[^-_%w/:.]', function(char)
                                     return string.format ("%%%02X", char:byte())
                                  end)
end

-- Create a new PhotoSet
function ViewbookAPI.createPhotoSet(params,propertyTable)
	--[[ response
		<photoset id="1234" url="http://username.viewboo.com/nameofset" />
	--]]

	propertyTable.loading = true;

	--info('Called ViewbookAPI.createPhotoSet with: ',params.title)
	-- prepare vars
	--params.auth_token = params.auth_token or prefs.token
	params.method = "flickr.photosets.create"

	-- GET API KEY FROM MEMORY
	local apiKey, sharedSecret, token = ViewbookAPI.getApiKeyAndSecret()
	params.api_key = apiKey
	params.auth_token = token
	params.api_sig = ViewbookAPI.makeApiSignature( params )
	
	-- prepare Post vars
	local postVars = "api_sig="..params.api_sig.."&auth_token="..params.auth_token.."&title="..urlencode(params.title).."&method="..params.method.."&api_key="..params.api_key

	-- Post it and handle the result
	-- This is an async task so all goes in the async call
	LrFunctionContext.postAsyncTaskWithContext( 'post ',
		function( context )
			result = LrHttp.post( "http://api.viewbook."..prefs.toplevel.."/services/rest/?"..postVars, "")
			--info("RESPONSE:", result)
			
			--info('ViewbookAPI.createPhotoSet Result:', result)
			local simpleXml = xmlElementToSimpleTable( result )
			if simpleXml.stat == 'ok' then
				--info('New PhotoSet ID: ', simpleXml.photoset.id)
				--add to property table
				propertyTable.photoset = simpleXml.photoset.id
				propertyTable.photoset_url = simpleXml.photoset.url
				--disable add to existing Photos
				propertyTable.addToPhotoset = false
				propertyTable.loading = false;
			else
				LrErrors.throwUserError( LOC( "$$$/Viewbook/Error/API/Upload=Viewbook API returned an error message (function upload, message ^1)",
									tostring( simpleXml.err and simpleXml.err.msg ) ) )
			end
	end )
end

--------------------------------------------------------------------------------
-- Request the photoset list
function ViewbookAPI.getUploadStatus(propertyTable)

	params = {}
	params.method = 'flickr.people.getUploadStatus' 

	-- Automatically add API key.
	local apiKey, sharedSecret = ViewbookAPI.getApiKeyAndSecret()
	
	if not params.api_key then
		params.api_key = apiKey
	end
	if prefs.token then
		params.auth_token = prefs.token
		--info("TOKEN", prefs.token)
	end
	
	-- Build up the URL for this function.
	params.api_sig = ViewbookAPI.makeApiSignature( params )
	local url = string.format( 'http://api.viewbook.'..prefs.toplevel..'/services/rest/?method=%s', assert( params.method ) )
	
	for name, value in pairs( params ) do
		if name ~= 'method' and value then  -- the 'and value' clause allows us to ignore false
			-- URL encode each of the params.
			value = string.gsub( value, '([^ !#-$&-*,%-%.0-;@-~])', function( c ) return string.format( '%%%02X', string.byte( c ) ) end )
			value = string.gsub( value, ' ', '+' )
			params[ name ] = value
			url = string.format( '%s&%s=%s', url, name, value )
		end
	end

	-- Call the URL and wait for response.
	--info( 'Calling Viewbook API via URL:', url )

	LrFunctionContext.postAsyncTaskWithContext( 'get task ',
		function( context )
			local response = LrHttp.get(url)
			--info ('API response XX', response )
			-- All responses are XML. Parse it now.
--[[			local simpleXml = xmlElementToSimpleTable( response )
			if simpleXml.stat == 'ok' then
				--info( 'Viewbook API returned status OK' )
			else
				warn( 'Viewbook API returned error', simpleXml.err and simpleXml.err.msg )
				LrErrors.throwUserError( LOC( "$$$/Viewbook/Error/API=Viewbook API returned an error message (function ^1, message ^2)",
									tostring( params.method ),
									tostring( simpleXml.err and simpleXml.err.msg ) ) )
			end
			--]]
		end)
end

--------------------------------------------------------------------------------

function ViewbookAPI.openAuthUrl()
	-- Request the frob that we need for authentication.
	local data = ViewbookAPI.callRestMethod{ method = 'flickr.auth.getFrob' }
	-- Get the frob from the response.
	local frob = assert( data.frob._value )
   -- Do the authentication. (This is not a standard REST call.)
	local apiKey, sharedSecret = ViewbookAPI.getApiKeyAndSecret()
	local authApiSig = ViewbookAPI.makeApiSignature{ perms = 'delete', frob = frob }
	local authURL = string.format( 'http://api.viewbook.'..prefs.toplevel..'/services/auth/?api_key=%s&perms=delete&frob=%s&api_sig=%s',
						apiKey, frob, authApiSig )
	LrHttp.openUrlInBrowser( authURL )
	return frob
end


--------------------------------------------------------------------------------

function ViewbookAPI.callRestMethod( params )

	-- Automatically add API key.
	local apiKey, sharedSecret = ViewbookAPI.getApiKeyAndSecret()
	
	if not params.api_key then
		params.api_key = apiKey
	end
	if prefs.token then
		params.auth_token = prefs.token
	end
	
	-- Build up the URL for this function.
	
	params.api_sig = ViewbookAPI.makeApiSignature( params )
	local url = string.format( 'http://api.viewbook.'..prefs.toplevel..'/services/rest/?method=%s', assert( params.method ) )
	
	for name, value in pairs( params ) do
		if name ~= 'method' and value then  -- the 'and value' clause allows us to ignore false
			-- URL encode each of the params.
			value = string.gsub( value, '([^ !#-$&-*,%-%.0-;@-~])', function( c ) return string.format( '%%%02X', string.byte( c ) ) end )
			value = string.gsub( value, ' ', '+' )
			params[ name ] = value
			url = string.format( '%s&%s=%s', url, name, value )
		end
	end

	--info("URL", url)

	-- Call the URL and wait for response.
	local response = LrHttp.get(url)

	--info("Response: ",response)

	-- All responses are XML. Parse it now.
	local simpleXml = xmlElementToSimpleTable( response )
	if simpleXml.stat == 'ok' then
		return simpleXml, response
	else
		warn( 'Viewbook API returned error', simpleXml.err and simpleXml.err.msg )
		LrErrors.throwUserError( LOC( "$$$/Viewbook/Error/API=Viewbook API returned an error message (function ^1, message ^2)",
							tostring( params.method ),
							tostring( simpleXml.err and simpleXml.err.msg ) ) )

	end

end

