--[[----------------------------------------------------------------------------

ViewbookUploadTask.lua
Upload photos to Viewbook

--------------------------------------------------------------------------------

VIEWBOOK B.V.
 Copyright 2012-2008 Viewbook B.V.
 All Rights Reserved.



------------------------------------------------------------------------------]]

	-- Lightroom API
local LrDialogs = import 'LrDialogs'
local LrErrors = import 'LrErrors'
local LrFileUtils = import 'LrFileUtils'
local LrHttp = import 'LrHttp'
local LrPathUtils = import 'LrPathUtils'

	-- Viewbook plug-in
require 'ViewbookAPI'


--============================================================================--

ViewbookUploadTask = {}

--------------------------------------------------------------------------------
-- Map Lightroom dialog property values to Viewbook API values.

local function booleanToNumber( value )
	return value and 1 or 0
end

local privacyToNumber = {
	private = 0,
	public = 1,
}

local safetyToNumber = {
	safe = 1,
	moderate = 2,
	restricted = 3,
}

local contentTypeToNumber = {
	photo = 1,
	screenshot = 2,
	other = 3,
}

--------------------------------------------------------------------------------

local function checkForPreviousUploads( exportSession )

	-- Check for photos that have been uploaded already.

	local existingViewbookIds = {}
			-- key: (LrPhoto)
			-- value: Viewbook photo ID from previous upload

	local numExisting = 0

	exportSession.catalog:withReadAccessDo( function()

		for photo in exportSession:photosToExport() do
	
			photo:withSettingsForPluginDo( 'com.viewbook.lightroom.export.flickr', function( settings )
	
				local flickrPhotoId = settings.photo_id
				if flickrPhotoId then
					numExisting = numExisting + 1
					existingViewbookIds[ photo ] = flickrPhotoId
				end
	
				return false -- hint that nothing was changed
		
			end )
	
		end
	
	end )

	-- If there were pre-existing photos, ask the user what to do about them.
	
	if numExisting > 0 then

		-- Ask user what to do about existing images.
		
		local response = LrDialogs.confirm(
							LOC "$$$/Viewbook/AlreadyUploaded/Error=Some of these images have been uploaded already. What would you like to do?",
							LOC "$$$/Viewbook/AlreadyUploaded/Hint=You can only replace images if you have paid for a Pro account on Viewbook.",
							LOC "$$$/Viewbook/AlreadyUploaded/ActionSkip=Skip",
							LOC "$$$/LrDialogs/Cancel=Cancel",
							LOC "$$$/Viewbook/AlreadyUploaded/ActionReplace=Replace" )
		
		if response == 'cancel' then
			LrErrors.throwCanceled()
		end
		
		if response == 'ok' then
		
			-- User said "Skip" these images. Remove them from the export manifest.
			
			for photo in pairs( existingViewbookIds ) do
				exportSession:removePhoto( photo )
			end
	
		end
	
	end
	
	-- Return the map of photos to existing Viewbook IDs.
		
	return existingViewbookIds

end

--------------------------------------------------------------------------------

function ViewbookUploadTask.processRenderedPhotos( functionContext, exportContext )

	-- Check for photos that have been uploaded already.
	
	local exportSession = exportContext.exportSession
	--local existingViewbookIds = checkForPreviousUploads( exportSession )

	-- Make a local reference to the export parameters.
	
	local exportParams = exportContext.propertyTable
	
	-- Set progress title.

	local nPhotos = exportSession:countRenditions()

	local progressScope = exportContext:configureProgress{
						title = nPhotos > 1
									and LOC( "$$$/Viewbook/Upload/Progress=Uploading ^1 photos to Viewbook", nPhotos )
									or LOC "$$$/Viewbook/Upload/Progress/One=Uploading one photo to Viewbook",
					}

	-- Save off uploaded photo IDs so we can take user to those photos later.
	
	local uploadedPhotoIds = {}

	-- Iterate through photo renditions.

	for i, rendition in exportContext:renditions{ stopIfCanceled = true } do
	
		-- Get next photo.

		local photo = rendition.photo
		local success, pathOrMessage = rendition:waitForRender()
		
		-- Check for cancellation again after photo has been rendered.
		
		if progressScope:isCanceled() then break end
		
		if success then

			-- Build up common metadata for this photo.
	
			local title, description
			
			photo.catalog:withReadAccessDo( function()

				title = photo:getFormattedMetadata 'title'
				if not title or #title == 0 then
					-- title = LrPathUtils.leafName( pathOrMessage )
				end
				
				description = photo:getFormattedMetadata 'caption'
			
			end )
			
			local tags = exportParams.addTags
				-- Viewbook will pick up LR keywords from XMP, so we don't need to merge them here.
				
			-- add to PhotoSet	
			local addToPhotoset = exportParams.addToPhotoset
			local photoset = exportParams.photoset 
			
			local is_public = privacyToNumber[ exportParams.privacy ]
			local is_friend = booleanToNumber( exportParams.privacy_friends )
			local is_family = booleanToNumber( exportParams.privacy_family )
			local safety_level = safetyToNumber[ exportParams.safety ]
			local content_type = contentTypeToNumber[ exportParams.contentType ]
			local hidden = exportParams.hideFromPublic and 2 or 1
			
			-- See if we previously uploaded this photo.
			
			local flickrPhotoId

--[[		TO DO: Hold off on this until we can replace photos...
			photo:withSettingsForPluginDo( 'com.viewbook.lightroom.export.flickr', function( settings )

				flickrPhotoId = settings.photo_id
	
				return false -- hint that nothing was changed
		
			end )
]]--

			-- Upload or replace the photo.
			
			if flickrPhotoId then
			
				error "TO DO: Not yet ready to replace existing photos on Viewbook."
			
			else
			 
				flickrPhotoId = ViewbookAPI.uploadPhoto{
										filePath = pathOrMessage,
										title = title,
										description = description,
										tags = tags,
										is_public = is_public,
										is_friend = is_friend,
										is_family = is_family,
										safety_level = safety_level,
										content_type = content_type,
										hidden = hidden,
										photoset_id = photoset,
									}				
				
				-- TO DO: Delete the individual photo file after completion.
				-- Note: This is just a better space-saving operation. LR
				-- will delete the folder of generated files at the end of the
				-- export since we requested that they be written to a temporary
				-- location.
	
				-- Record this Viewbook ID with the photo so we know to replace
				-- instead of upload.
				
				--[[ photo.catalog:withCatalogDo( function()

					photo:withSettingsForPluginDo( 'com.viewbook.lightroom.export.flickr', function( settings )
				
						settings.photo_id = flickrPhotoId

					end )
				
				 end ]]--
			
				-- When done with photo, delete temp file. There is a cleanup step that happens later,
				-- but this will help manage space in the event of a large upload.
				
				LrFileUtils.delete( pathOrMessage )
				
			end
			
			-- Remember this in the list of photos we uploaded.

			uploadedPhotoIds[ #uploadedPhotoIds + 1 ] = flickrPhotoId
		
		end
		
	end

	-- Take user to uploaded photos.
	
		-- TO DO: What happens if there are 1000 photos uploaded?
	
		--[[
	if #uploadedPhotoIds > 0 then
		local newPhotosURL = 'http://www.flickr.com/tools/uploader_edit.gne?ids=' .. table.concat( uploadedPhotoIds, ',' )
		LrHttp.openUrlInBrowser( newPhotosURL )
	end
	]]
	
	if #uploadedPhotoIds > 0 then
		LrDialogs.message( "Your export is finished!", "Login or reload your Image Manager to view the images.", "info" )
	end
	

end
