--[[----------------------------------------------------------------------------

ViewbookUser.lua
Viewbook user account management

--------------------------------------------------------------------------------

VIEWBOOK B.V.
 Copyright 2012-2008 Viewbook B.V.
 All Rights Reserved.



------------------------------------------------------------------------------]]

	-- Lightroom SDK
local LrBinding = import 'LrBinding'
local LrDialogs = import 'LrDialogs'
local LrFunctionContext = import 'LrFunctionContext'

local prefs = import 'LrPrefs'.prefsForPlugin()


--Logging
--[[
local logger = import 'LrLogger'( 'ViewbookAPILog' )
local debug, info, warn, err = logger:quick( 'debug', 'info', 'warn', 'err' )
--Viewbook
logger:enable( "logfile" )
--]]

--============================================================================--

ViewbookUser = {}

--------------------------------------------------------------------------------

local function storedCredentialsAreValid()

	return prefs.token and prefs.nsid and prefs.username and prefs.fullname

end

--------------------------------------------------------------------------------

local function notLoggedIn( propertyTable )

	propertyTable.token = nil
	
	prefs.nsid = nil
	prefs.username = nil
	prefs.fullname = nil
	prefs.token = nil
	-- Viewbook
	prefs.password = nil

	propertyTable.accountStatus = LOC "$$$/Viewbook/AccountStatus/NotLoggedIn=Not logged in! Click Login to start using this Plugin."
	propertyTable.loginButtonTitle = LOC "$$$/Viewbook/LoginButton/NotLoggedIn=Log In"
	propertyTable.loginButtonEnabled = true
	propertyTable.validAccount = false

end

--------------------------------------------------------------------------------

local doingLogin = false

function ViewbookUser.login( propertyTable )

	if doingLogin then return end
	doingLogin = true

	LrFunctionContext.postAsyncTaskWithContext( 'Viewbook login',
	function( context )

		-- Clear any existing login info.
	
		notLoggedIn( propertyTable )

		propertyTable.accountStatus = LOC "$$$/Viewbook/AccountStatus/LoggingIn=Logging in..."
		propertyTable.loginButtonEnabled = false
		
		-- Make sure login is valid when done, or is marked as invalid.
		
		context:addCleanupHandler( function()

			doingLogin = false

			if not storedCredentialsAreValid() then
				notLoggedIn( propertyTable )
			end
			
			-- Hrm. New API doesn't make it easy to show what operation failed.
			-- LrDialogs.message( LOC "$$$/Viewbook/LoginFailed=Failed to log in." )

		end )
		
		-- Make sure we have an API key.
		
		ViewbookAPI.getApiKeyAndSecret()

		-- Show request for authentication dialog.
	
		local authRequestDialogResult = LrDialogs.confirm(
			LOC "$$$/Viewbook/AuthRequestDialog/Message=Lightroom needs your permission to upload images to Viewbook.",
			LOC "$$$/Viewbook/AuthRequestDialog/HelpText=If you click Authorize, you will be taken to a web page in your web browser where you can log in. When you're finished, return to Lightroom to complete the authorization.",
			LOC "$$$/Viewbook/AuthRequestDialog/AuthButtonText=Authorize",
			LOC "$$$/LrDialogs/Cancel=Cancel" )
	
		if authRequestDialogResult == 'cancel' then
			return
		end
	
		-- Request the frob that we need for authentication.
		
		propertyTable.accountStatus = LOC "$$$/Viewbook/AccountStatus/WaitingForViewbook=Waiting for response from viewbook.com..."

		require 'ViewbookAPI'
		local frob = ViewbookAPI.openAuthUrl()
	
		local waitForAuthDialogResult = LrDialogs.confirm(
			LOC "$$$/Viewbook/WaitForAuthDialog/Message=Return to this window once you've authorized Lightroom on viewbook.com.",
			LOC "$$$/Viewbook/WaitForAuthDialog/HelpText=Once you've granted permission for Lightroom (in your web browser), click the Done button below.",
			LOC "$$$/Viewbook/WaitForAuthDialog/DoneButtonText=Done",
			LOC "$$$/LrDialogs/Cancel=Cancel" )
	
		if waitForAuthDialogResult == 'cancel' then
			return
		end
	
		-- User has OK'd authentication. Get the user info.
		
		propertyTable.accountStatus = LOC "$$$/Viewbook/AccountStatus/WaitingForViewbook=Waiting for response from viewbook.com..."

		local data = ViewbookAPI.callRestMethod{ method = 'flickr.auth.getToken', frob = frob }
		
		-- Now we can read the Viewbook user credentials. Save off to prefs.
	
		local auth = assert( data.auth )
	
		prefs.nsid = auth.user.nsid
		prefs.username = auth.user.username
		prefs.fullname = auth.user.fullname
		prefs.token = auth.token._value
		propertyTable.token = prefs.token
		
	end )

end

--------------------------------------------------------------------------------

function ViewbookUser.verifyLogin( propertyTable )

	-- Observe changes to prefs and update status message accordingly.

	local function updateStatus()
	
		if storedCredentialsAreValid() then

			local displayUserName = prefs.fullname
			if not displayUserName or #displayUserName == 0 then
				displayUserName = prefs.username
			end
			propertyTable.accountStatus = LOC( "$$$/Viewbook/AccountStatus/LoggedIn=Logged in as ^1", displayUserName )
			propertyTable.loginButtonTitle = LOC "$$$/Viewbook/LoginButton/LoggedIn=Switch User?"
			propertyTable.loginButtonEnabled = true
			
			-- only enable export when photoset is choosen
			propertyTable.LR_canExport = false
			
			propertyTable.validAccount = true
			
			propertyTable.LR_cantExportBecause = LOC "$$$/Viewbook/ExportDialog/NoLogin=You need to select or create an Album first."
			
			ViewbookAPI.getUploadStatus(propertyTable)
			
		else
			notLoggedIn( propertyTable )
		end
	
	end
	
	propertyTable:addObserver( 'token', updateStatus )
	updateStatus()
end






